//***********************************************************************
//	SDIB32GL.H
//	
//	SELNA DIB 32bit Graphic Libraries
//
//	ver 0.95 (build2004/1/26)	
//	WebSite: http://selna.hp.infoseek.co.jp
//	E-Mail: selna_id@yahoo.co.jp
//
//	Copyright(C) 2004 SELNA. All rights reserved.
//**********************************************************************/

#ifndef _INC_ISDIB32GL
#define _INC_ISDIB32GL

#include <windows.h>


/*----------------------------------------------------------------------------*
|	GUIDS used by SDIB32GL objects
*----------------------------------------------------------------------------*/
DEFINE_GUID( CLSID_SDib32GLX86,		0xc9d37ff0,0x4117,0x4fe4,0xae,0xb6,0x3e,0x91,0x29,0xae,0xa7,0x4d);
DEFINE_GUID( CLSID_SDib32GLPMMX,	0x1c9a1da6,0xa308,0x48f1,0x92,0xfc,0x30,0x97,0x8c,0xb5,0xd1,0x41);
DEFINE_GUID( IID_ISDib32GL,			0x49591e1c,0x6539,0x4af9,0xa5,0xd8,0xd0,0xdd,0xf0,0xeb,0xb3,0xe5);


/*----------------------------------------------------------------------------*
|	SDIB32GL Structures
*----------------------------------------------------------------------------*/
struct ISDib32GL;
typedef DWORD SDIB32GLCOLOR, FAR *LPSDIB32GLCOLOR;
typedef ISDib32GL FAR *LPSDIB32GL;

// SDIB32GLsNZtH[}bg\(܂)
typedef union _SDIB32GLARGB{
	struct{
		BYTE blue;		// 
		BYTE green;		// 
		BYTE red;		// 
		BYTE alpha;		// 
	};
	SDIB32GLCOLOR argb;
}SDIB32GLARGB;


/*----------------------------------------------------------------------------*
|	SDIB32GL ERRORS
*----------------------------------------------------------------------------*/
#define SDIB32GLERR_SDIB32GLALREADYCREATED 		MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x200 )
#define SDIB32GLERR_DIBNOTCREATED				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x201 )
#define SDIB32GLERR_MEMORYDCNOTCREATED			MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x202 )
#define SDIB32GLERR_FILENOTFOUND				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x203 )
#define SDIB32GLERR_FILENOTCREATED				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x204 )
#define SDIB32GLERR_NOTINITIALIZED				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x205 )
#define SDIB32GLERR_INVALIDPOSITION				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x206 )
#define SDIB32GLERR_INVALIDOBJECT				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x207 )
#define SDIB32GLERR_INVALIDPARAMS				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x208 )
#define SDIB32GLERR_UNSUPPORTEDFILE				MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x300 )
#define SDIB32GLERR_UNSUPPORTEDBITCOUNT			MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x301 )
#define SDIB32GLERR_UNSUPPORTEDFORMAT			MAKE_HRESULT( SEVERITY_ERROR,  FACILITY_ITF, 0x302 )
#define SDIB32GLERR_OUTOFMEMORY					E_OUTOFMEMORY
#define SDIB32GLS_ROTATEBLT64BIT				MAKE_HRESULT( SEVERITY_SUCCESS,  FACILITY_ITF, 0x350 )


/*----------------------------------------------------------------------------*
|	SDIB32GL UTILITY MACROS
*----------------------------------------------------------------------------*/
#define SDIB32GL_ARGB( a, r, g, b )		((DWORD)((((a)&0xFF)<<24)|(((r)&0xFF)<<16)|(((g)&0xFF)<<8)|((b)&0xFF)))
#define SDIB32GL_RGBA( r, g, b, a )		SDIB32GL_ARGB( a, r, g, b ) 
#define SDIB32GL_XRGB( r, g, b )		SDIB32GL_ARGB( 0xFF, r, g, b )
#define SDIB32GL_SET_RED( r )			SDIB32GL_ARGB( 0xFF, r, 0, 0 )
#define SDIB32GL_SET_GREEN( g )			SDIB32GL_ARGB( 0xFF, 0, g, 0 )
#define SDIB32GL_SET_BLUE( b )			SDIB32GL_ARGB( 0xFF, 0, 0, b )
#define SDIB32GL_SET_ALPHA( a )			SDIB32GL_ARGB( a, 0xFF, 0xFF, 0xFF )
#define SDIB32GL_RGB SDIB32GL_XRGB
#define SDIB32GL_GET_REVERSE_ALPHA( argb )	(255-SDIB32GL_GET_ALPHA( argb ))	
#define SDIB32GL_GET_ALPHA( argb )		((BYTE)((argb)>>24))
#define SDIB32GL_GET_RED( argb )		((BYTE)((argb)>>16))
#define SDIB32GL_GET_GREEN( argb )		((BYTE)((argb)>>8))
#define SDIB32GL_GET_BLUE( argb )		((BYTE)((argb))) 
#define SDIB32GL_A360To512( a )			(((a) << 9 ) / 360 )
#define SDIB32GL_A512To360( a )			(((a) * 360) >> 9 )
#define SDIB32GL_WHITE					0xFFFFFFFF
#define SDIB32GL_BLACK					0xFF000000
#define SDIB32GL_RED					0xFFFF0000
#define SDIB32GL_GREEN					0xFF00FF00
#define SDIB32GL_BLUE					0xFF0000FF
#define SDIB32GL_SEPIA					0xFFF0C891			// SDIB32GL_XRGB(240,200,145)


/*----------------------------------------------------------------------------*
|	SDib32GL Interface
*----------------------------------------------------------------------------*/
DECLARE_INTERFACE_( ISDib32GL, IUnknown )
{
	/*** IUnknown methods ***/
	STDMETHOD( QueryInterface)( REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject) PURE;
	STDMETHOD_( ULONG, AddRef )( void ) PURE;
	STDMETHOD_( ULONG, Release)( void ) PURE;

	/*** ISDib32GL methods ***/
	STDMETHOD( LoadFromFile )( const char* lpszFileName, SDIB32GLCOLOR ColorKey=SDIB32GL_XRGB(0,0,0), HDC hdc=NULL ) PURE;
	STDMETHOD( CreateBuffer )( int nWidth, int nHeight, SDIB32GLCOLOR FillColor=SDIB32GL_XRGB(0,0,0), HDC hdc=NULL ) PURE;
	STDMETHOD( DuplicateBuffer )( LPSDIB32GL lpSDIB32GLOriginal ) PURE;
	STDMETHOD( SaveAsFile )( const char* lpszFileName ) PURE;
	STDMETHOD( Blt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( ColorKeyBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( AlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255 ) PURE;
	STDMETHOD( ColorKeyAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255 ) PURE;
	STDMETHOD( AddBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( AddAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255 ) PURE;
	STDMETHOD( SubBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( SubAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255 ) PURE;
	STDMETHOD( AlphaChannelBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( AlphaChannelAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255 ) PURE;
	STDMETHOD( RotateBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateColorKeyBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateColorKeyAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateAddBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateAddAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateSubBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateSubAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateAlphaChannelBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( RotateAlphaChannelAlphaBlendBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, BYTE Alpha=255, int nAngle=0, LPPOINT lpCenterPt=NULL ) PURE;
	STDMETHOD( ColorKeyToAlpha )( void ) PURE;
	STDMETHOD( RGBToAlphaBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( RToAlphaBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( GToAlphaBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( BToAlphaBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( AlphaToAlphaBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL ) PURE;
	STDMETHOD( ColorKeyToColorBlt )( LPSDIB32GL lpSrcSDIB32GL, int nDstX, int nDstY, LPSIZE lpDstSize=NULL, LPPOINT lpSrcPt=NULL, LPSIZE lpSrcSize=NULL, SDIB32GLCOLOR Color=SDIB32GL_XRGB(255,255,255) ) PURE;
	STDMETHOD( SetColorKey )( SDIB32GLCOLOR ColorKey ) PURE;
	STDMETHOD( SetClipArea )( int nX, int nY, int nW, int nH ) PURE;
	STDMETHOD( SetClipRect )( LPRECT lpRect ) PURE;
	STDMETHOD( GetClipRect )( LPRECT lpRect ) PURE;
	STDMETHOD_( int, GetWidth )( void ) PURE;
	STDMETHOD_( int, GetHeight )( void ) PURE;
	STDMETHOD_( HDC, GetDC )( void ) PURE;
	STDMETHOD_( HBITMAP, GetBitmap )( void ) PURE;
	STDMETHOD_( LPSDIB32GLCOLOR, GetBuffer )( void ) PURE;
	STDMETHOD_( LPSDIB32GLCOLOR, GetScanLine )( int nLine ) PURE;
	STDMETHOD_( LPSDIB32GLCOLOR*, GetScanLineBuffer )( void ) PURE;
	STDMETHOD_( SDIB32GLCOLOR, GetColorKey )( void ) PURE;
	STDMETHOD_( DWORD, GetPixels )( void ) PURE;
	STDMETHOD_( DWORD, GetSizeImage )( void ) PURE;
	STDMETHOD( GetFileName )( char* lpszFileName ) PURE;
	STDMETHOD( SetPixel )( int x, int y, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD_( SDIB32GLCOLOR, GetPixel )( int x, int y ) PURE;
	STDMETHOD( Clear )( SDIB32GLCOLOR Color=SDIB32GL_WHITE ) PURE;
	STDMETHOD( Rectangle )( int x, int y, int nWidth, int nHeight, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( RectangleAlpha )( int x, int y, int nWidth, int nHeight, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( RectangleAdd )( int x, int y, int nWidth, int nHeight, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( RectangleSub )( int x, int y, int nWidth, int nHeight, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( Line )( int x0, int y0, int x1, int y1, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( LineAlpha )( int x0, int y0, int x1, int y1, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( LineAdd )( int x0, int y0, int x1, int y1, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( LineSub )( int x0, int y0, int x1, int y1, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( Ellipse )( int x0, int y0, int rx, int ry, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( EllipseAlpha )( int x0, int y0, int rx, int ry, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( EllipseAdd )( int x0, int y0, int rx, int ry, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( EllipseSub )( int x0, int y0, int rx, int ry, SDIB32GLCOLOR Color ) PURE;
	STDMETHOD( Nega )( int x, int y, int nWidth, int nHeight ) PURE;
	STDMETHOD( Mono )( int x, int y, int nWidth, int nHeight, BYTE Strength ) PURE;
	STDMETHOD( Sepia )( int x, int y, int nWidth, int nHeight, BYTE Strength, SDIB32GLCOLOR Color  ) PURE;
	STDMETHOD( Bright )( int x, int y, int nWidth, int nHeight, int nBright ) PURE;
	STDMETHOD( Mosaic )( int x, int y, int nWidth, int nHeight, int nBlockWidth, int nBlockHeight ) PURE;
};

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

HRESULT
WINAPI
SDIB32GL_CreateInstance(
	REFCLSID rclsid,
	REFIID riid,
	LPVOID* ppv
	);

HRESULT
WINAPI
SDIB32GL_CreateBuffer(
	int nWidth,
	int nHeight,
	SDIB32GLCOLOR FillColor,
	HDC hdc,
	REFCLSID rclsid,
	REFIID riid,
	LPVOID* lplpSDIB32GL
	);

HRESULT
WINAPI
SDIB32GL_LoadFromFile(
	const char* lpszFileName,
	SDIB32GLCOLOR ColorKey,
	HDC hdc,
	REFCLSID rclsid,
	REFIID riid,
	LPVOID* lplpSDIB32GL
	);

HRESULT
WINAPI
SDIB32GL_DuplicateBuffer(
	LPVOID lpSDIB32GLOriginal,
	REFCLSID rclsid,
	REFIID riid,
	LPVOID* lplpSDIB32GLDuplicate
	);

DWORD
WINAPI
SDIB32GL_GetErrorText(
	HRESULT hr,
	char* pBuffer,
	int nBufferMax
	);

DWORD
WINAPI
SDIB32GL_GetVersion( void );

ULONG
WINAPI
SDIB32GL_GetDllRefCount( void );

BOOL
WINAPI
SDIB32GL_IsMMX( void );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _INC_ISDIB32GL */
